% Script for plotting the variation in installed power for each thermal
% unit dependant on the electricity and gas price as these vary.

% -------------------------------------------------------------------
% Optimum_design_variable_elprice_and_gasprice must be run in advance
% -------------------------------------------------------------------

optimum_design_variable_elprice_and_gasprice;

% ------------------------------------------------------------
installed_capacities = 0;
surfplots = 0;
lineplots = 0;
    lineplotHP = 0;
    lineplotEB = 0;
    lineplotPB = 1;
yearly_costs = 0;
% ------------------------------------------------------------

if(installed_capacities)
    if(surfplots)
        subplot(1,3,1)
        surf(elprice_low:elprice_step:elprice_high,gasprice_low:gasprice_step:gasprice_high,plant_matrix_HP);
        ylabel('El price [NOK/kWh]');
        xlabel('Gas price [NOK/kWh]');
        zlabel('Installed capacity [kW]')
        legend('Heat pump power');
        legend('Location','north')
        axis('tight');
        
        subplot(1,3,2)
        surf(elprice_low:elprice_step:elprice_high,gasprice_low:gasprice_step:gasprice_high,plant_matrix_EB);
        ylabel('El price [NOK/kWh]');
        xlabel('Gas price [NOK/kWh]');
        zlabel('Installed capacity [kW]')
        legend('Electrical boiler power');
        legend('Location','north')
        axis('tight');
        
        subplot(1,3,3)
        surf(elprice_low:elprice_step:elprice_high,gasprice_low:gasprice_step:gasprice_high,plant_matrix_PB);
        ylabel('El [NOK/kWh]');
        xlabel('Gas price [NOK/kWh]');
        zlabel('Installed capacity [kW]')
        legend('Propane burner power');
        legend('Location','north')
        axis('tight');
    elseif(lineplots)
        if(lineplotHP)
        plot(0.2:0.05:0.6,plant_matrix_HP')
        legend('Electricity price 0.2 NOK/kWh','Electricity price 0.25 NOK/kWh','Electricity price 0.3 NOK/kWh','Electricity price 0.35 NOK/kWh','Electricity price 0.4 NOK/kWh','Electricity price 0.45 NOK/kWh','Electricity price 0.5 NOK/kWh','Electricity price 0.55 NOK/kWh','Electricity price 0.6 NOK/kWh')
        ylabel('Gas price [NOK/kWh]')
        zlabel('Capacity [kW]')
        title('Installed capacity for heat pump [kW] at varying energy prices [NOK/kWh]')
        set(gca,'XTick',[])
        axis('tight')
        elseif(lineplotEB)
        plot(0.2:0.05:0.6,plant_matrix_EB')
        legend('Electricity price 0.2 NOK/kWh','Electricity price 0.25 NOK/kWh','Electricity price 0.3 NOK/kWh','Electricity price 0.35 NOK/kWh','Electricity price 0.4 NOK/kWh','Electricity price 0.45 NOK/kWh','Electricity price 0.5 NOK/kWh','Electricity price 0.55 NOK/kWh','Electricity price 0.6 NOK/kWh')
        ylabel('Gas price [NOK/kWh]')
        zlabel('Capacity [kW]')
        title('Installed capacity for electrical boiler [kW] at varying energy prices [NOK/kWh]')
        set(gca,'XTick',[])
        axis('tight')
        elseif(lineplotPB)
        plot(0.2:0.05:0.6,plant_matrix_PB')
        legend('Electricity price 0.2 NOK/kWh','Electricity price 0.25 NOK/kWh','Electricity price 0.3 NOK/kWh','Electricity price 0.35 NOK/kWh','Electricity price 0.4 NOK/kWh','Electricity price 0.45 NOK/kWh','Electricity price 0.5 NOK/kWh','Electricity price 0.55 NOK/kWh','Electricity price 0.6 NOK/kWh')
        ylabel('Gas price [NOK/kWh]')
        zlabel('Capacity [kW]')
        title('Installed capacity for propane boiler [kW] at varying energy prices [NOK/kWh]')
        set(gca,'XTick',[])
        axis('tight')
        end
    end
elseif(yearly_costs)
    surf(elprice_low:elprice_step:elprice_high,gasprice_low:gasprice_step:gasprice_high,plant_matrix_cost);
    xlabel('Gas price [NOK/kWh]');
    ylabel('Electricity price [NOK/kWh]');
end