% Script which sets basic variables and generates a power demand for use in
% optimization
% -----------------------------------------------------------------------

clear
clearvars -global
global power_demand;
global test_production_matrix;
global interest_rate;
global gasprice_constant;
gasprice_constant = 0.4;
global elprice_constant;
elprice_constant = 0.6;
% Diverse variable for systemet:
coincidence_factor_heating = 0.5; % samtidighetsfaktor for romoppvarming
p_heating_installed = 4746; % total installert romoppvarmingseffekt
coincidence_factor_hot_water = 0.5; % samtidighetsfaktor for tappevann
p_hot_water = 817; % total installert tappevannseffekt
coincidence_factor_street_heating = 0.75; % samtidighetsfaktor for gatevarme
p_street_heating_installed = 2585; % total installert gatevarmeeffekt
global failsafe_power_supply_factor;
failsafe_power_supply_factor = 0.0;
cp_water = 4.18; % oppgitt i kJ/(kg*K)
electricity_price_data = load('elprice_2013_hourly.txt'); % laster inn strmpris fra txt-fil
electricity_price = electricity_price_data(:,3)/10000; % kr/kWh.
electricity_grid_fee = 0.14;
electricity_price = electricity_price + electricity_grid_fee;
global electricity_co2_emission;
electricity_co2_emission = 0.002/1000; % antall tonn CO2 sluppet ut per kWh produsert ved opphavet
gas_price = ones(1,8760)*0.4; % setter gassprisen konstant lik 40 re/kWh
global gas_co2_emission;
gas_co2_emission = 0.215/1000; % antall tonn CO2 sluppet ut per kWh brennverdi for gass
global co2_tax;
co2_tax = 60; % kostnad for utslipp av et tonn CO2. Her satt til 60 kroner som tilsvarer 7,2 EUR.
interest_rate = 0.05; % Renteniv for bruk til investeringsanalyse
global storage_max;
storage_max = 0;%5000*4.18*10/3600; % 5000 liters lagringstank
global storage_loss_factor;
storage_loss_factor = 0.005;
global waste_heat_connected
waste_heat_connected = 0;


heating_temperature = 17; % vre utendrstemperatur hvor oppvarming er ndvendig
loss_factor_to_ground = 0.9; % varmetapsfaktor, angitt som andelen av produsert energi som blir levert til kunden
temp_return = 60; % prosjektert returtemperatur i distribusjonsnettet
temp_supply = 95; % prosjektert turtemperatur i distribusjonsnettet
temp_diff = temp_supply-temp_return; % prosjektert temperaturforskjell
max_flow_variation = 0.5; % oppgis i andel massestrm som kan reguleres ned
cop_lower_limit = 0; % nedre grense for hvilken COP varmepumpen skal opereres ved
hot_water_daily_variation = [0.50 0.50 0.50 0.57 0.57 0.57 0.57 0.57 ...
                            0.71 0.96 1 0.89 0.82 0.82 0.82 0.85 0.89 ...
                            0.85 0.78 0.71 0.64 0.57 0.50 0.50]; % variasjon i varmtvannsbehov over dgnet
%                            alle elementer svarer til behovet for
%                            foregende time, alts er det frste elementet
%                            verdien for tidsrommet 23-00

global heat_pump_control_cost electrical_boiler_control_cost propane_burner_control_cost heat_pump_part_load_limit electrical_boiler_part_load_limit propane_burner_part_load_limit
heat_pump_power = 1204;
heat_pump_temp_lift = 10;
heat_pump_control_cost = 0;
heat_pump_part_load_limit = 0.1;
global heat_pump_max_limit;
heat_pump_max_limit = 1;
global heat_pump_improvement_with_waste_heat;
heat_pump_improvement_with_waste_heat = 0;

electrical_boiler_power = 3300;
electrical_boiler_control_cost = 0;
electrical_boiler_part_load_limit = 0;
global electrical_boiler_max_limit;
electrical_boiler_max_limit = 1;

propane_burner_power = 3500;
propane_burner_control_cost = 0;
propane_burner_part_load_limit = 0.2;
global propane_burner_max_limit;
propane_burner_max_limit = 1;

global C_specific_HP % Investment cost per installed kW
C_specific_HP = 3750;
global maintenance_HP % Percentage of initial investment cost used for maintenance each year
maintenance_HP = 0.03;
global lifespan_HP
lifespan_HP = 20;

% Electrical boiler
global C_specific_EB % Investment cost per installed kW
C_specific_EB = 850;
global maintenance_EB % Percentage of initial investment cost used for maintenance each year
maintenance_EB = 0.03;
global lifespan_EB
lifespan_EB = 20;

% Propane burner
global C_specific_PB % Investment cost per installed kW
C_specific_PB = 900;
global maintenance_PB % Percentage of initial investment cost used for maintenance each year
maintenance_PB = 0.03;
global lifespan_PB
lifespan_PB = 20;

% Laster inn vrdata for Hammerfest
weather_data = load('timedata_holt_2013.txt');
max_outside_temp_difference = heating_temperature - (min(weather_data(:,3)));
p_heating_specific = p_heating_installed*coincidence_factor_heating/max_outside_temp_difference;
% varmebehovsfaktor for kundemassen per kelvin temperaturforskjell i
% uteluften fra en standardtemperatur (heating_temperature) - angitt i kW

% beregner effektbehov til tappevann basert p daglig variasjonskurve
power_hot_water = zeros(8760,1)+p_hot_water*coincidence_factor_hot_water;
for i = 1 : length(power_hot_water);
    k = mod(i,24)+1;
    switch(k)
        case 1
            power_hot_water(i) = power_hot_water(i)*hot_water_daily_variation(k);
        case 2
            power_hot_water(i) = power_hot_water(i)*hot_water_daily_variation(k);
        case 3
            power_hot_water(i) = power_hot_water(i)*hot_water_daily_variation(k);
        case 4
            power_hot_water(i) = power_hot_water(i)*hot_water_daily_variation(k);
        case 5
            power_hot_water(i) = power_hot_water(i)*hot_water_daily_variation(k);
        case 6
            power_hot_water(i) = power_hot_water(i)*hot_water_daily_variation(k);
        case 7
            power_hot_water(i) = power_hot_water(i)*hot_water_daily_variation(k);
        case 8
            power_hot_water(i) = power_hot_water(i)*hot_water_daily_variation(k);
        case 9
            power_hot_water(i) = power_hot_water(i)*hot_water_daily_variation(k);
        case 10
            power_hot_water(i) = power_hot_water(i)*hot_water_daily_variation(k);
        case 11
            power_hot_water(i) = power_hot_water(i)*hot_water_daily_variation(k);
        case 12
            power_hot_water(i) = power_hot_water(i)*hot_water_daily_variation(k);
        case 13
            power_hot_water(i) = power_hot_water(i)*hot_water_daily_variation(k);
        case 14
            power_hot_water(i) = power_hot_water(i)*hot_water_daily_variation(k);
        case 15
            power_hot_water(i) = power_hot_water(i)*hot_water_daily_variation(k);
        case 16
            power_hot_water(i) = power_hot_water(i)*hot_water_daily_variation(k);
        case 17
            power_hot_water(i) = power_hot_water(i)*hot_water_daily_variation(k);
        case 18
            power_hot_water(i) = power_hot_water(i)*hot_water_daily_variation(k);
        case 19
            power_hot_water(i) = power_hot_water(i)*hot_water_daily_variation(k);
        case 20
            power_hot_water(i) = power_hot_water(i)*hot_water_daily_variation(k);
        case 21
            power_hot_water(i) = power_hot_water(i)*hot_water_daily_variation(k);
        case 22
            power_hot_water(i) = power_hot_water(i)*hot_water_daily_variation(k);
        case 23
            power_hot_water(i) = power_hot_water(i)*hot_water_daily_variation(k);
        case 24
            power_hot_water(i) = power_hot_water(i)*hot_water_daily_variation(k);
    end
end

% beregner effektbehov til oppvarming
power_heating = zeros(8760,1);
for i = 1 : length(power_heating);
    if(weather_data(i,3) <= heating_temperature) % tar hensyn til oppvarmingstemperaturen,
                                                 % slik at man ikke fr negative bidrag til effektbehovet
        power_heating(i) = p_heating_specific * (heating_temperature - weather_data(i,3));
        % oppvarmingsbehov beregnes ut fra spesifikt oppvarmingsbehov og
        % temperaturforskjell
    end
end

% legger til effektbehov til tappevann
power_demand = power_heating + power_hot_water;

% legger til effektbehov til gateoppvarming
% antar at dette kun er ndvendig ved utetemperatur lavere enn -5 grader
% celcius
p_street_heating_specific = p_street_heating_installed*coincidence_factor_street_heating/(-5-min(weather_data(:,3)));
power_street_heating = zeros(8760,1);
for i = 1:8760
    if weather_data(i,3) <= -5
        power_street_heating(i) = p_street_heating_specific*(-5-weather_data(i,3));
    end
end
power_demand = power_demand + power_street_heating;

% tar hyde for varmetap i distribusjonsnettet
power_demand = power_demand/loss_factor_to_ground;
power_demand = power_demand/0.9297;