% Function which optimizes the the plant operation for a specific hour
% --------------------------------------------------------------------

function [x,fval] = hourly_optimization(power_demandIN,el_priceIN,propane_priceIN,VPmaxIN,EBmaxIN,PBmaxIN)

% Define global variables for use in object function without
% transfer as input variables
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
global VPmax EBmax PBmax EL_PRICE PROPANE_PRICE POWER_DEMAND_HOUR;
VPmax = VPmaxIN;
EBmax = EBmaxIN;
PBmax = PBmaxIN;
EL_PRICE = el_priceIN;
PROPANE_PRICE = propane_priceIN;
POWER_DEMAND_HOUR = power_demandIN;
global heat_pump_part_load_limit;
global propane_burner_part_load_limit;

% Enumeration of target variables
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
variables = {'VP','EB','PB'};
N = length(variables);
for v = 1:N
    eval([variables{v},' = ',num2str(v),';']);
end

% Set lower bounds for target variables
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
lb = zeros(size(variables));
lb([VP,PB]) = [heat_pump_part_load_limit*VPmax,propane_burner_part_load_limit*PBmax];

% Set upper bounds for target variables
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
ub([VP,EB,PB]) = [VPmax,EBmax,PBmax];

% Define linear inequalities
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


% Define linear equalities
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
Aeq = zeros(1,N); beq = zeros(1,1);
Aeq(1,[VP,EB,PB]) = [1,1,1]; beq(1) = POWER_DEMAND_HOUR;

% Define nonlinear inequality function
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


% Set options for the minimization application
options = optimset('Display','off');
%options = optimset('Display','iter','PlotFcns','optimplotx','PlotFcns','optimplotfval');
[x, fval] = fmincon(@objecfun,[VPmax,EBmax,PBmax],[],[],Aeq,beq,lb,ub,[],options);

end