% Function which calculates the operational and financial cost of the plant
% throughout an entire year, using a slightly less accurate operational
% strategy than @objecfun
% ------------------------------------------------------------------------

function cost = plant_operation(p_data)

global running_energy_price_simulation;
global power_demand;
global test_production_matrix;
global interest_rate;
global electricity_price_iter;
global gas_price_iter;
global electricity_co2_emission;
global gas_co2_emission;
global heat_pump_part_load_limit
global co2_tax;
global electrical_boiler_part_load_limit;
global propane_burner_part_load_limit;
global heat_pump_improvement_with_waste_heat;
global elprice_constant;
global gasprice_constant;

Qhp_max = p_data(1);
Qhp_min = p_data(4);
%Qhp_min = Qhp_max * 0.25;
%Qhp_min = heat_pump_part_load_limit;
Qeb_max = p_data(2);
Qeb_min = p_data(5); % = 0;
%Qeb_min = electrical_boiler_part_load_limit;
Qpb_max = p_data(3);
Qpb_min = p_data(6);
%Qpb_min = Qpb_max * 0.25;
%Qpb_min = propane_burner_part_load_limit;
temp_demand = power_demand;

if(running_energy_price_simulation)
    el_price = electricity_price_iter;
    gas_price = gas_price_iter;
else
    el_price = elprice_constant; % Kilde NVE
    gas_price = gasprice_constant; % Kilde NVE. Her er det lagt til 0.08 for transport og infrastruktur
end


% Termisk energilagring
% ------------------------------------
global storage_max;
global storage_loss_factor;
global storage_status
storage_status = zeros(8761,2);
global storage_priority_interval;
global max_storage_time;
global max_storage_prediction_time
global is_running_storage_optimization
storage_priority_interval = 24;
max_storage_time = 24;
max_storage_prediction_time = 24;
storage_max = 5000*4.18*10/3600; % 5000 liters lagringstank kg*kJ/kgK*K/(s/h)
heat_pump_power = Qhp_max;
for i = 1:8760
    storage_status(i,3) = power_demand(i);
    % Control of Thermal Energy Storage
    hourly_storage_loss = 0;
    if(storage_status(i,1) > 0)
        storage_status(i+1,1) = max(0,storage_status(i,1)-storage_max*storage_loss_factor);
        hourly_storage_loss = storage_status(i,1)-storage_status(i+1,1);
    end
    if (power_demand(i) < heat_pump_power) && (storage_status(i+1,1) < storage_max) && (max(power_demand(i:min(i+max_storage_prediction_time,8760)))>heat_pump_power)
        available_storage = storage_max-storage_status(i+1,1);
        storage_status(i+1,1) = storage_status(i+1,1) + min(heat_pump_power-power_demand(i),storage_max-storage_status(i+1,1));
        storage_status(i,2) = storage_status(i+1,1) - storage_status(i,1) + hourly_storage_loss;
        power_demand(i) = power_demand(i) + min((heat_pump_power-power_demand(i)),available_storage);
    elseif ((power_demand(i) > heat_pump_power) && (storage_status(i+1,1) > 0)) && (power_demand(i) == max(power_demand(i:min(i+storage_priority_interval,8760))))
        available_stored_heat = storage_status(i+1,1);
        storage_status(i+1,1) = storage_status(i+1,1) - min(available_stored_heat,power_demand(i)-heat_pump_power);
        storage_status(i,2) = storage_status(i+1,1) - storage_status(i,1) + hourly_storage_loss;
        power_demand(i) = power_demand(i) - min(available_stored_heat,power_demand(i)-heat_pump_power);
    elseif (max(power_demand(i:min(i+max_storage_time,8760)))<=heat_pump_power)
        available_stored_heat = storage_status(i+1,1);
        storage_status(i+1,1) = storage_status(i+1,1) - available_stored_heat;
        storage_status(i,2) = storage_status(i+1,1) - available_stored_heat;
        power_demand(i) = power_demand(i) - available_stored_heat;
    end
    storage_status(i,4) = power_demand(i);
end
% -----------------------------------------

% Effektfordeling
% ----------------------------------------
Qhp = zeros(8760,1);
Qeb = zeros(8760,1);
Qpb = zeros(8760,1);
COP = zeros(8760,1);
EFFeb = ones(8760,1)*0.98;
EFFpb = zeros(8760,1);
for i = 1:8760
    hourly_demand = power_demand(i);
    if hourly_demand >= Qhp_min
        Qhp(i) = min(hourly_demand,Qhp_max);
        hourly_demand = hourly_demand - Qhp(i);
    end
    if hourly_demand >= Qpb_min
        if ((min(hourly_demand,Qpb_max)/propane_burner_efficiency(min(hourly_demand,Qpb_max)/Qpb_max))*(gas_price + gas_co2_emission*co2_tax) + ((hourly_demand-(min(hourly_demand,Qpb_max)))/EFFeb(i))*(el_price + electricity_co2_emission*co2_tax)) < ...
                ((((min(hourly_demand,Qeb_max))/EFFeb(i))*(el_price + electricity_co2_emission*co2_tax)) + ((hourly_demand-min(hourly_demand,Qeb_max))/propane_burner_efficiency(min(hourly_demand,Qpb_max)/Qpb_max))*(gas_price + gas_co2_emission*co2_tax))
            Qpb(i) = min(hourly_demand,Qpb_max);
            Qeb(i) = hourly_demand-min(hourly_demand,Qpb_max);
        else
            Qeb(i) = min(hourly_demand,Qeb_max);
            Qpb(i) = hourly_demand - min(hourly_demand,Qeb_max);
        end
    else
        Qeb(i) = hourly_demand;
    end
%     if hourly_demand > Qeb_max
%         if hourly_demand-Qeb_max < Qpb_min
%             Qpb(i) = Qpb_min;
%         else
%             Qpb(i) = min(Qpb_max,hourly_demand-Qeb_max);
%         end
%         hourly_demand = hourly_demand - Qpb(i);
%     end
%     Qeb(i) = max(hourly_demand,Qeb_min);
    
    if(Qhp_max==0)
        COP(i,1) = 1;
    else
        COP(i,1) = heat_pump_efficiency(Qhp(i)/Qhp_max)*(1+heat_pump_improvement_with_waste_heat);
    end
    if(Qpb_max==0)
        EFFpb(i,1) = 1;
    else
        EFFpb(i,1) = propane_burner_efficiency(Qpb(i)/Qpb_max);
    end
end


% Operational cost of each unit
consumption_HP = Qhp./COP;
consumption_EB = Qeb./EFFeb;
consumption_PB = Qpb./EFFpb;

% Financial cost of each unit
% Heat pump
global C_specific_HP % Investment cost per installed kW
global maintenance_HP % Percentage of initial investment cost used for maintenance each year
global lifespan_HP
C_financial_HP = (((interest_rate*(1+interest_rate)^lifespan_HP)/((1+interest_rate)^lifespan_HP-1)) + maintenance_HP) * C_specific_HP*Qhp_max;


% Electrical boiler
global C_specific_EB % Investment cost per installed kW
global maintenance_EB % Percentage of initial investment cost used for maintenance each year
global lifespan_EB
C_financial_EB = (((interest_rate*(1+interest_rate)^lifespan_EB)/((1+interest_rate)^lifespan_EB-1)) + maintenance_EB) * C_specific_EB*Qeb_max;


% Propane burner
global C_specific_PB % Investment cost per installed kW
global maintenance_PB % Percentage of initial investment cost used for maintenance each year
global lifespan_PB
C_financial_PB = (((interest_rate*(1+interest_rate)^lifespan_PB)/((1+interest_rate)^lifespan_PB-1)) + maintenance_PB) * C_specific_PB*Qpb_max;

% Total yearly energy cost
cost = sum((el_price + electricity_co2_emission*co2_tax) * (consumption_HP + consumption_EB) + (gas_price + gas_co2_emission*co2_tax) * consumption_PB) + C_financial_HP + C_financial_EB + C_financial_PB;


test_production_matrix(:,4) = power_demand;
test_production_matrix(:,1) = Qhp; % Delivered heat from heat pump
test_production_matrix(:,5) = COP; % Heat pump COP
test_production_matrix(:,2) = Qeb; % Delivered heat from electrical boiler
test_production_matrix(:,6) = EFFeb; % Electrical boiler efficiency
test_production_matrix(:,3) = Qpb; % Delivered heat from gas boiler
test_production_matrix(:,7) = EFFpb; % Gas boiler efficiency
test_production_matrix(:,8) = el_price * (consumption_HP + consumption_EB); % Electricity cost
test_production_matrix(:,9) = gas_price * consumption_PB; % Gas cost
test_production_matrix(:,11) = electricity_co2_emission*co2_tax * (consumption_HP + consumption_EB); % Electricity CO2 tax
test_production_matrix(:,10) = electricity_co2_emission*(consumption_HP + consumption_EB); % Electricity CO2 emissions
test_production_matrix(:,13) = gas_co2_emission*co2_tax * consumption_PB; % Gas CO2 tax
test_production_matrix(:,12) = gas_co2_emission*consumption_PB; % Gas CO2 emissions
test_production_matrix(:,14) = (el_price + electricity_co2_emission*co2_tax) * (consumption_HP + consumption_EB) + (gas_price + gas_co2_emission*co2_tax) * consumption_PB; % Hourly operations cost

% test_production_matrix(:,1) = power_demand;
% test_production_matrix(:,2) = Qhp; % Delivered heat from heat pump
% test_production_matrix(:,3) = COP; % Heat pump COP
% test_production_matrix(:,4) = Qeb; % Delivered heat from electrical boiler
% test_production_matrix(:,5) = EFFeb; % Electrical boiler efficiency
% test_production_matrix(:,6) = Qpb; % Delivered heat from gas boiler
% test_production_matrix(:,7) = EFFpb; % Gas boiler efficiency
% test_production_matrix(:,8) = el_price * (consumption_HP + consumption_EB); % Electricity cost
% test_production_matrix(:,9) = gas_price * consumption_PB; % Gas cost
% test_production_matrix(:,10) = (C_financial_HP + C_financial_EB + C_financial_PB)/8760; % Financial cost
% test_production_matrix(:,11) = electricity_co2_emission*co2_tax * (consumption_HP + consumption_EB); % Electricity CO2 tax
% test_production_matrix(:,12) = electricity_co2_emission*(consumption_HP + consumption_EB); % Electricity CO2 emissions
% test_production_matrix(:,13) = gas_co2_emission*co2_tax * consumption_PB; % Gas CO2 tax
% test_production_matrix(:,14) = gas_co2_emission*consumption_PB; % Gas CO2 emissions
% test_production_matrix(:,15) = (el_price + electricity_co2_emission*co2_tax) * (consumption_HP + consumption_EB) + (gas_price + gas_co2_emission*co2_tax) * consumption_PB; % Hourly operations cost
% test_production_matrix(:,16) = (el_price + electricity_co2_emission*co2_tax) * (consumption_HP + consumption_EB) + (gas_price + gas_co2_emission*co2_tax) * consumption_PB + test_production_matrix(:,10); % Total hourly cost

%fprintf('Energy from HP: %d\tEnergy from EB: %d\tEnergy from PB: %d\n',sum(Qhp),sum(Qeb),sum(Qpb));
power_demand = temp_demand;
end