% Function which runs the design optimization, and then runs the
% operational optimization for each hour given the design from the
% optimization
% ---------------------------------------------------------------------

clear
clearvars -global
global source_active;
source_active = zeros(8761,3);
global storage_max;
global storage_loss_factor;
global storage_status
storage_status = zeros(8761,2);
global current_hour;
global electricity_co2_emission;
global gas_co2_emission;
global co2_tax;
data_initialization;
% For kjring med ulike strrelser p lagringstank
storage_max = 5000*4.18*10/3600; % 5000 liters lagringstank kg*kJ/kgK*K/(s/h)
storage_priority_interval = 24;
max_storage_time = 24;
max_storage_prediction_time = 24;
% For kjring med full 2oo3-dekning
% global failsafe_power_supply_factor;
% failsafe_power_supply_factor = 1;
% For kjring med kt COP for varmepumpe
% global heat_pump_improvement_with_waste_heat;
% heat_pump_improvement_with_waste_heat = 0.15;

% Kommenteres ut hvis man nsker kjring med designet fra Hammerfest Energi
plant_opt_fmincon;
heat_pump_power = p_size(1);
electrical_boiler_power = p_size(2);
propane_burner_power = p_size(3);
% Direkte valg av dimensjonering
% ------------------------------
% heat_pump_power = 2457;
% electrical_boiler_power = 834;
% propane_burner_power = 2106;
% ------------------------------

power_production = zeros(8760,3); production_cost = zeros(8760,1);
power_production(:,4) = power_demand;
% For kjring med konstant elektrisitetspris
electricity_price = ones(1,8760)*0.6; % setter elektrisitetsprisen konstant lik 60 re/kWh
% ------------------------
control_alternative = 3;
% ------------------------

if control_alternative == 1
    % Driftsalternativ 1, konstant massestrm og variabel delta T.
    mass_flow = zeros(1,8760);
    delivered_temperature = zeros(1,8760);
    for i = 1:8760
        mass_flow(i) = power_demand(i)/(cp_water*temp_diff);
        % Mulighet for  legge inn nedre eller vre grense for massestrm
        delivered_temperature(i) = temp_return+power_demand(i)/(cp_water*mass_flow(i));
    end
elseif control_alternative == 2
    % Alternativ 2, variabel delta T og konstant massestrm
    mass_flow = ones(1,8760);
    mass_flow = mass_flow*max(power_demand)/(cp_water*temp_diff);
    delivered_temperature = zeros(1,8760);
    for i = 1:8760
        delivered_temperature(i) = temp_return+power_demand(i)/(cp_water*mass_flow(i));
    end
elseif control_alternative == 3
    % Alternativ 3, variabel delta T og variabel massestrm.
    mass_flow = zeros(1,8760);
    delivered_temperature = zeros(1,8760);
    max_needed_flow = max(power_demand)/(temp_diff*cp_water);
    for i = 1:8760
        if power_demand(i) <= heat_pump_power
            mass_flow(i)=power_demand(i)/(cp_water*heat_pump_temp_lift);
            % beregner massestrm ndvendig for  dekke effektbehovet ved varmepumpens maksimale temperaturkning
            mass_flow(i)=max(power_demand(i)/(cp_water*heat_pump_temp_lift), ...
                (1-max_flow_variation)*heat_pump_power/(cp_water*heat_pump_temp_lift));
            % setter massestrm lik en minimumsverdi dersom verdien beregnet i forrige linje er lavere enn denne
        elseif power_demand(i) <= max_needed_flow*cp_water*(heat_pump_temp_lift)
            mass_flow(i) = power_demand(i)/(cp_water*heat_pump_temp_lift);
        else
            mass_flow(i) = max_needed_flow;
        end
        delivered_temperature(i) = temp_return + power_demand(i)/(mass_flow(i)*cp_water);
    end
end

tic;
for i = 1:8760
    current_hour = i;
    %disp(i);
    storage_status(i,3) = power_demand(i);
    % Control of Thermal Energy Storage
    hourly_storage_loss = 0;
    if(storage_status(i,1) > 0)
        storage_status(i+1,1) = max(0,storage_status(i,1)-storage_max*storage_loss_factor);
        hourly_storage_loss = storage_status(i,1)-storage_status(i+1,1);
    end
    if (power_demand(i) < heat_pump_power) && (storage_status(i+1,1) < storage_max) && (max(power_demand(i:min(i+max_storage_prediction_time,8760)))>heat_pump_power)
        available_storage = storage_max-storage_status(i+1,1);
        storage_status(i+1,1) = storage_status(i+1,1) + min(heat_pump_power-power_demand(i),storage_max-storage_status(i+1,1));
        storage_status(i,2) = storage_status(i+1,1) - storage_status(i,1) + hourly_storage_loss;
        power_demand(i) = power_demand(i) + min((heat_pump_power-power_demand(i)),available_storage);
    elseif ((power_demand(i) > heat_pump_power) && (storage_status(i+1,1) > 0)) && (power_demand(i) == max(power_demand(i:min(i+storage_priority_interval,8760))))
        available_stored_heat = storage_status(i+1,1);
        storage_status(i+1,1) = storage_status(i+1,1) - min(available_stored_heat,power_demand(i)-heat_pump_power);
        storage_status(i,2) = storage_status(i+1,1) - storage_status(i,1) + hourly_storage_loss;
        power_demand(i) = power_demand(i) - min(available_stored_heat,power_demand(i)-heat_pump_power);
    elseif (max(power_demand(i:min(i+max_storage_time,8760)))<=heat_pump_power)
        available_stored_heat = storage_status(i+1,1);
        storage_status(i+1,1) = storage_status(i+1,1) - available_stored_heat;
        storage_status(i,2) = storage_status(i+1,1) - available_stored_heat;
        power_demand(i) = power_demand(i) - available_stored_heat;
    end
    storage_status(i,4) = power_demand(i);
    
    % Distribution of heat production
    % Calculate best production distribution alternative if all sources are available
    [power_alt1, prod_cost_alt1] = hourly_optimization(power_demand(i),electricity_price(i),gas_price(i),heat_pump_power,electrical_boiler_power,propane_burner_power);
    if abs((sum(power_alt1)-power_demand(i)))>0.01
        prod_cost_alt1 = prod_cost_alt1*100;
    end
    prod_cost_alt1 = prod_cost_alt1 + heat_pump_control_cost*abs(source_active(i,1)-power_alt1(1)>0.1);
    prod_cost_alt1 = prod_cost_alt1 + propane_burner_control_cost*abs(source_active(i,3)-power_alt1(3)>0.1);
    % Calculate best production distribution alternative if propane burner is shut down
    [power_alt2, prod_cost_alt2] = hourly_optimization(power_demand(i),electricity_price(i),gas_price(i),heat_pump_power,electrical_boiler_power,0);
    if abs((sum(power_alt2)-power_demand(i)))>0.01
        prod_cost_alt2 = prod_cost_alt2*100;
    end
    prod_cost_alt2 = prod_cost_alt2 + heat_pump_control_cost*abs(source_active(i,1)-power_alt2(1)>0.1);
    prod_cost_alt2 = prod_cost_alt2 + propane_burner_control_cost*abs(source_active(i,3)-power_alt2(3)>0.1);
    % Calculate best production distribution alternative if heat pump is shut down
    [power_alt3, prod_cost_alt3] = hourly_optimization(power_demand(i),electricity_price(i),gas_price(i),0,electrical_boiler_power,propane_burner_power);
    if abs((sum(power_alt3)-power_demand(i)))>0.01
        prod_cost_alt3 = prod_cost_alt3*100;
    end
    prod_cost_alt3 = prod_cost_alt3 + heat_pump_control_cost*abs(source_active(i,1)-power_alt3(1)>0.1);
    prod_cost_alt3 = prod_cost_alt3 + propane_burner_control_cost*abs(source_active(i,3)-power_alt3(3)>0.1);
    % Sort alternatives based on production cost
    sorted_alternatives = sortrows([power_alt1,prod_cost_alt1;power_alt2,prod_cost_alt2;power_alt3,prod_cost_alt3],4);
    %disp(sorted_alternatives);
    % Allocate the power production distribution to the power production
    % matrix
    %if sum(sorted_alternatives(1,1:3)) >= sum(sorted_alternatives(2,1:3))
    
    power_production(i,1:3) = sorted_alternatives(1,1:3) ; production_cost(i) = sorted_alternatives(1,4);
    
    power_production(i,15) = power_demand(i)-sum(power_production(i,1:3)); % differanse mellom levert varme og behov
    
    source_active(i+1,1) = power_production(i,1)>=0.01;
    source_active(i+1,2) = power_production(i,2)>=0.01;
    source_active(i+1,3) = power_production(i,3)>=0.01;
    if(mod(i,1000)==0)
        disp(i);
    end
end
toc;

heat_pump_cycles = 0;
propane_burner_cycles = 0;
electrical_boiler_cycles = 0;

for i = 1:8760
    k = source_active(i,:) ~= source_active(i+1,:);
    heat_pump_cycles = heat_pump_cycles + k(1);
    electrical_boiler_cycles = electrical_boiler_cycles + k(2);
    propane_burner_cycles = propane_burner_cycles + k(3);
end


for i = 1:8760
    power_production(i,5) = heat_pump_efficiency(power_production(i,1)/heat_pump_power); % Heat pump COP
    power_production(i,6) = 0.98; % Electrical boiler efficiency
    power_production(i,7) = propane_burner_efficiency(power_production(i,3)/propane_burner_power); % Propane burner efficiency
    power_production(i,8) = electricity_price(i)*(power_production(i,1)/power_production(i,5)+power_production(i,2)/power_production(i,6)); % Electricity cost
    power_production(i,9) = gas_price(i)*power_production(i,3)/power_production(i,7); % Gas cost
    power_production(i,10) = (power_production(i,1)/power_production(i,5)+power_production(i,2)/power_production(i,6))*electricity_co2_emission; % Electricity CO2 emission
    power_production(i,11) = power_production(i,10)*co2_tax; % Electricity CO2 cost
    power_production(i,12) = (power_production(i,3)/power_production(i,6))*gas_co2_emission; % Gas CO2 emission
    power_production(i,13) = power_production(i,12)*co2_tax; % Gas CO2 cost
    power_production(i,14) = power_production(i,8)+power_production(i,9)+power_production(i,11)+power_production(i,13); % Total driftskostnad
end

save('fullYearlySimulationConstantElpriceOptDesignStorage5000')